// Maurice Ribble 
// April-7-2015
// Open Source, licensed under a Creative Commons Attribution 3.0 License (http://creativecommons.org/licenses/by-sa/3.0/)
// Compiled with Arduino Software 1.0.6 (http://arduino.cc) plus the attiny module (https://github.com/damellis/attiny)

// ATtiny45 Reset Pin Fuse Note
// This uses the reset pin (PB5) on the ATtiny45 because otherwise the ATtiny45 didn't have enough IO pins.  To use the
// reset pin as an IO pin you must first load this program.  Then change the fuses so reset is an IO.  Setting the reset 
// pin to an IO pin prevents the chip from being reprogrammed unless you have a HVPP programmer which uses 12V to reset
// the fuses.
// This sets the fuses:
// cd D:\apps\arduino-1.0.6\hardware\tools\avr\bin
// avrdude.exe -C D:\apps\arduino-1.0.6\hardware\tools\avr\etc\avrdude.conf -c usbtiny -p attiny45 -U lfuse:w:0x62:m -U hfuse:w:0x5f:m -U efuse:w:0xff:m

// REVISIONS:
// Initial Version

// Configuration flags
#define ATTINY45

#define UPDATE_HZ       500 /* Frequency for updating motor */
#define DEAD_ZONE_LOW   20  /* Pot position where speed control assumes motor is off */
#define DEAD_ZONE_HIGH  4   /* Pot position where speed control assumes motor is off */

// Define digital/analog pins
#ifdef ATTINY45
  #define MOT_EN_PIN            5
  #define MOT_A_PIN             3
  #define MOT_B_PIN             4
  #define PEDAL_ENABLE_PIN      0
  #define PEDAL_SWITCH_PIN      1
  #define SPEED_APIN            1
#else
  #define MOT_EN_PIN            8
  #define MOT_A_PIN             6
  #define MOT_B_PIN             9
  #define PEDAL_ENABLE_PIN      18
  #define PEDAL_SWITCH_PIN      19
  #define SPEED_APIN            0
#endif

byte g_spVal = 128;   // Speed value
byte g_spValPrev = 128; // Previous speed value

////////////////////////////////////////////////////////////////////////////////////////////////////
// digitalReadFast
////////////////////////////////////////////////////////////////////////////////////////////////////
inline byte digitalReadFast(byte p)
{
#ifdef ATTINY45
  return bitRead(PINB, p);
#else
  return digitalRead(p);
#endif
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// digitalWriteFastHigh
////////////////////////////////////////////////////////////////////////////////////////////////////
inline void digitalWriteFastHigh(byte p)
{
#ifdef ATTINY45
  bitSet(PORTB, p);
#else
  digitalWrite(p, HIGH);
#endif
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// digitalWriteFastLow
////////////////////////////////////////////////////////////////////////////////////////////////////
inline void digitalWriteFastLow(byte p)
{
#ifdef ATTINY45
  bitClear(PORTB, p);
#else
  digitalWrite(p, LOW);
#endif
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// setAnalogReadSpeed
////////////////////////////////////////////////////////////////////////////////////////////////////
void setAnalogReadSpeed()
{
  // ADPS2  ADPS1  ADPS0  Div Factor
  // 0      0      0      2
  // 0      0      1      2          // Fastest
  // 0      1      0      4
  // 0      1      1      8
  // 1      0      0      16
  // 1      0      1      32
  // 1      1      0      64
  // 1      1      1      128        // Slowest (default)
    
  // Default set by wiring is 128 (1 MHz/128 = 8 Khz)
  // Since a converstion takes 13 clocks the sample rate is 8Khz/13 = 630 Hz or about 1.59 ms
  // By setting the div factor to 8 things are 16 times faster (0.10 ms)
  ADCSRA &= ~(1<<ADPS2);
  ADCSRA |= (1<<ADPS1);
  ADCSRA |= (1<<ADPS0);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// setup - Main program init (this is just how Arduino code works)
////////////////////////////////////////////////////////////////////////////////////////////////////
void setup()
{
  setAnalogReadSpeed();
  
  pinMode(MOT_EN_PIN, OUTPUT);
  pinMode(MOT_A_PIN, OUTPUT);
  pinMode(MOT_B_PIN, OUTPUT);
#ifdef ATTINY45
  pinMode(PEDAL_ENABLE_PIN, INPUT_PULLUP);
  pinMode(PEDAL_SWITCH_PIN, INPUT_PULLUP);
#else
  pinMode(PEDAL_ENABLE_PIN, INPUT);
  pinMode(PEDAL_SWITCH_PIN, INPUT);
  pinMode(MOT_ENABLE_PIN, OUTPUT);
#endif
  delay(100);  // Let things settle
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// loop - Main program loop (this is just how Arduino code works)
////////////////////////////////////////////////////////////////////////////////////////////////////
void loop()
{
  // On ATtiny45
  //   PEDAL_ENABLE_PIN == HIGH is pedal attached
  //   PEDAL_SWITCH_PIN == LOW is not pressed (opposite on atmega328 version)
  if (digitalReadFast(PEDAL_ENABLE_PIN) == HIGH)
  {
#ifdef ATTINY45
    static byte offOn = 0;
    if (digitalReadFast(PEDAL_SWITCH_PIN) == LOW)
    { 
      setMotor(128);  // Force motor off while pedal is pressed
      offOn = !offOn;
      do
      {
        delay(10);  // Debounce
      }while(digitalReadFast(PEDAL_SWITCH_PIN) == LOW);
    }
#else
    static byte offOn = 0;
    if (digitalRead(PEDAL_SWITCH_PIN) == HIGH)
    { 
      setMotor(128);  // Force motor off while pedal is pressed
      offOn = !offOn;
      do
      {
        delay(10);  // Debounce
      }while(digitalRead(PEDAL_SWITCH_PIN) == HIGH);
    }
#endif

    if ( !offOn )
    {
      g_spVal = 128;
    }
  }

  setMotor(g_spVal);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// setMotor - Set speed and direction of the motor
// val is 0-255 (0 is full speed counter clockwise, 128 is off, and 255 is full speed clockwise)
////////////////////////////////////////////////////////////////////////////////////////////////////
void setMotor(byte val)
{
  if (val <= 127)  // Forwards
  {
    val = abs(int(val)-int(127));
    val = (val < DEAD_ZONE_LOW) ? 0 : val;
    val = (val > 127-DEAD_ZONE_HIGH) ? 127 : val;
    pulseMotor(MOT_A_PIN, MOT_B_PIN, val);
  }
  else             // Backwards
  {
    val -= 128;
    val = (val < DEAD_ZONE_LOW) ? 0 : val;
    val = (val > 127-DEAD_ZONE_HIGH) ? 127 : val;
    pulseMotor(MOT_B_PIN, MOT_A_PIN, val);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// pulseMotor - Does a high/low pulse to control the motor
////////////////////////////////////////////////////////////////////////////////////////////////////
void pulseMotor(byte lowPin, byte highPin, byte val)
{
  unsigned long delayTime;
  static unsigned long highTime = 1000000000/UPDATE_HZ/127 * val / 1000;
  static unsigned long lowTime  = 1000000000/UPDATE_HZ/127 * (127-val) / 1000;
  byte runFloatMath = (abs(int(g_spVal)-int(g_spValPrev)) > 4) || (g_spVal==127) || (g_spVal==0);

  if (val == 0)  // Motor off
  {
    g_spVal = analogRead(SPEED_APIN)>>2;
    digitalWriteFastLow(MOT_EN_PIN);
    return;
  }

  if (val != 127)
  {
    delayTime = micros();
    if (runFloatMath)
    {
      lowTime  = 1000000000/UPDATE_HZ/127 * (127-val) / 1000;
      g_spValPrev = g_spVal;
    }
    delayTime += lowTime;

    digitalWriteFastLow(MOT_EN_PIN);
    if (highTime < lowTime)
    {
      g_spVal = analogRead(SPEED_APIN)>>2;
    }
    while(micros() < delayTime);
  }

  delayTime = micros();
  if (runFloatMath)
  {
    highTime = 1000000000/UPDATE_HZ/127 * val / 1000;
    g_spValPrev = g_spVal;
  }
  delayTime += highTime;
  
  digitalWriteFastHigh(MOT_EN_PIN);
  digitalWriteFastLow(lowPin);
  digitalWriteFastHigh(highPin);
  if (highTime >= lowTime)
  {
    g_spVal = analogRead(SPEED_APIN)>>2;
  }
  while(micros() < delayTime);

  /*  for(int i=0; i<40; ++i)
  {
    digitalWriteFastHigh(MOT_EN_PIN);
    digitalWriteFastLow(lowPin);
    digitalWriteFastHigh(highPin);
    delay(highTime/1000);
    delayMicroseconds(highTime%1000);
  
    if (lowTime)
    {
      digitalWriteFastLow(MOT_EN_PIN);
      delay(lowTime/1000);
      delayMicroseconds(lowTime%1000);
    }
  }*/
}


